﻿import com.GameInterface.Game.Character;
import com.GameInterface.Utils;
import Enums.AttackDefensiveLevel;
import Enums.AttackOffensiveLevel;
import Enums.AttackResultType;
import com.GameInterface.Log;

/**
 * ...
 * @author Xxx
 */
class Statistics
{
	static public var Timeout:Number = 8;
	static public var Smooth:Number = 8;

	var m_ClientCharacter:Character;
	
	public var DamageIdleTime:Number = 8.0;
	
	public var damageDealt:Number = 0;
	public var damageObtained:Number = 0;
	private var _damageTime:Number = 0.0;
	private var _damagePerSec:Number = 0;
	public var damageLast:Number = 0;
	
	public var damageDealtHits:Number = 0;
	public var damageObtainedHits:Number = 0;
	
	public var damageCritDealt:Number = 0;
	public var damageCritHits:Number = 0;
	private var _damageCritRate:Number = 0;
	
	private var _damagePenRate:Number = 0;
	public var damagePenHits:Number = 0;
	public var damagePenDealt:Number = 0;

	public var damageSelfBlocked:Number = 0;
	public var damageTargetBlocked:Number = 0;
	public var damageGlanceDone:Number = 0;
	public var damageGlanceRecieved:Number = 0;
	public var damageSelfEvade:Number = 0;
	public var damageTargetEvade:Number = 0;

	/*
	public var healDealt:Number = 0;
	public var healHits:Number = 0;
	public var healCritAmount:Number = 0;
	public var healCritHits:Number = 0;
	private var _healTime:Number = 0.0;
	private var _healPerSec:Number = 0;
	public var healLast:Number = 0;
	*/
	
	public function Statistics() 
	{
		m_ClientCharacter = Character.GetClientCharacter();
		_healingDeal = new StatInfo();
		_healingObtained = new StatInfo();
		_damageDealt = new StatInfo();
	}
	
	public function update()
	{
		// update DPS
		if (_damageTime > 0.0)
			_damagePerSec = damageDealt / _damageTime;
		
		if(damageDealtHits > 0)
		{
			_damageCritRate = damageCritHits / damageDealtHits;
			_damagePenRate = damagePenHits / damageDealtHits;
		}
	}
	
	public function Add(rec:StatRecord):Void
	{
		if (rec.isSupport())
			processSupport(rec);
		else 
			processDamage(rec);
	}
	
	private function processDamage(rec:StatRecord):Void
	{
		var time:Number = timeNow();
		var elapsed:Number = time - damageLast;
		
		if (rec.damage == 0 && rec.absorb == 0 && rec.attackResultType != _global.Enums.AttackResultType.e_AttackType_Evade)
			return;
		
		_damageTime += elapsed;
		
		if (!isDamaging(elapsed))
		{
			_damageTime = 0;
			damageDealt = 0;
			damageObtained = 0;
			damageDealtHits = 0;
			damageObtainedHits = 0;
			damageCritDealt = 0;
			_damageCritRate = 0;
			damageCritHits = 0;
			damagePenDealt = 0;
			_damagePenRate = 0;
			damagePenHits = 0;
			damageSelfBlocked = 0;
			damageTargetBlocked = 0;
			damageGlanceDone = 0;
			damageGlanceRecieved = 0;
			damageSelfEvade = 0;
			damageTargetEvade = 0;
		}
		
		if (rec.isInflict())
		{
			_damageDealt.update(rec, time);
			processDamageInflict(rec);
			damageDealt = _damageDealt.total;
		}
		else if(rec.isObtained())
			processDamageObtained(rec);

		damageLast = time;
	}
	
	private var _damageDealt: StatInfo;
	
	private function processDamageInflict(rec:StatRecord):Void 
	{
		damageDealtHits++;
		damageDealt += (rec.damage - rec.absorb);
		
		// crit
		if (rec.attackResultType == _global.Enums.AttackResultType.e_AttackType_CriticalHit)
		{
			damageCritDealt += (rec.damage - rec.absorb);
			damageCritHits++;
		}
		
		// evaded attacks
		if (rec.attackResultType == _global.Enums.AttackResultType.e_AttackType_Evade)
			damageTargetEvade++;
		
		// penetration
		if (rec.attackDefensiveLevel == _global.Enums.AttackDefensiveLevel.e_DefensiveLevel_Penetrated)
		{
			damagePenDealt += (rec.damage - rec.absorb);
			damagePenHits++;
		}
		
		// blocked
		if (rec.attackDefensiveLevel == _global.Enums.AttackDefensiveLevel.e_DefensiveLevel_Blocked)
			damageTargetBlocked++;
		
		// glace
		if (rec.attackOffensiveLevel == _global.Enums.AttackOffensiveLevel.e_OffensiveLevel_Glancing)
			damageGlanceDone++;
	}
	
	private function processDamageObtained(rec:StatRecord):Void 
	{
		damageObtainedHits++;
		damageObtained += (rec.damage - rec.absorb);
		
		// evaded attacks
		if (rec.attackResultType == _global.Enums.AttackResultType.e_AttackType_Evade)
			damageSelfEvade++;
		
		// blocked
		if (rec.attackDefensiveLevel == _global.Enums.AttackDefensiveLevel.e_DefensiveLevel_Blocked)
			damageSelfBlocked++;
		
		// glace
		if (rec.attackOffensiveLevel == _global.Enums.AttackOffensiveLevel.e_OffensiveLevel_Glancing)
			damageGlanceRecieved++;
	}
	
	private var _healingDeal: StatInfo;
	private var _healingObtained: StatInfo;
	
	private function processSupport(rec:StatRecord):Void 
	{
		var time:Number = timeNow();
		
		if (rec.damage == 0 && rec.absorb == 0)
			return;
		
		//Log.Error("HealStats","Healing: " + DPSUtils.dumpObj(rec, ""));
		if (rec.isInflict())
		{
			if(rec.isHeal())
				_healingDeal.update(rec, time);
		}
		else if(rec.isObtained())
		{
			_healingObtained.update(rec, time);
		}
	}
	
	private function isDamaging(elapsed:Number):Boolean
	{
		return elapsed < DamageIdleTime;
	}
	
	private function timeNow():Number 
	{
		return Utils.GetNormalTime();
	}
	
	public function get damageTime():Number 
	{
		return Math.floor(_damageDealt.time);
	}
	
	public function get damagePerSec():Number 
	{
		return Math.floor(_damageDealt.getPersec());
	}

	public function get damageAvrg():Number 
	{
		return Math.floor(_damageDealt.avarage);
	}
	
	public function get healDealt():Number 
	{
		return Math.floor(_healingDeal.total);
	}

	public function get healTime():Number 
	{
		return Math.floor(_healingDeal.time);
	}
	
	public function get healAvrg():Number 
	{
		return Math.floor(_healingDeal.avarage);
	}

	public function get healPerSec():Number 
	{
		return Math.floor(_healingDeal.getPersec());
	}
	
	public function get damageCritDealtPercent():Number 
	{
		return Math.floor(damageCritDealt / (damageDealt + 0.01) * 100);
	}
	
	public function get damageCritRate():Number 
	{
		return Math.floor(_damageCritRate * 100);
	}
	
	public function get damagePenRate():Number 
	{
		return Math.floor(_damagePenRate * 100);
	}
	
	public function get damagePenDealtPercent():Number 
	{
		return Math.floor(damagePenDealt / (damageDealt + 0.01) * 100);
	}
	
	public function get damageSelfBlockedPercent():Number
	{
		return getPercent(damageSelfBlocked, damageObtainedHits);
	}
	
	public function get damageTargetBlockedPercent():Number
	{	
		return getPercent(damageTargetBlocked, damageDealtHits);
	}

	public function get damageGlanceDonePercent():Number
	{
		return getPercent(damageGlanceDone, damageDealtHits);
	}
	
	public function get damageGlanceRecievedPercent():Number
	{
		return getPercent(damageGlanceRecieved, damageObtainedHits);
	}
	
	public function get damageSelfEvadePercent():Number 
	{
		return getPercent(damageSelfEvade, damageObtainedHits);
	}
	
	public function get damageTargetEvadePercent():Number 
	{
		return getPercent(damageTargetEvade, damageDealtHits);
	}
	
	public function getPercent(val:Number, max:Number):Number 
	{
		if(max > 0)
			return Math.floor(val / max * 100);
		return 0;
	}
	
}